;;; -*- Mode: Scheme -*- 

;;	Author: Sam Rushing <rushing@nightmare.com>
;;	$Id: socket.scm,v 1.1 1997/03/10 07:43:16 rushing Exp $

;;; socket interface for gambit-c scheme

(c-declare "#include \"scm_socket.h\"")

(define-macro (snarf-constant constant)
  `(define ,(string->symbol constant)
     ((c-lambda
       ()
       int
       ,(string-append
	 (string-append "___result = " constant) ";")))))

(snarf-constant "AF_INET")
(snarf-constant "AF_UNIX")
(snarf-constant "SOCK_STREAM")
(snarf-constant "SOCK_DGRAM")
(snarf-constant "EINPROGRESS")
(snarf-constant "EWOULDBLOCK")
(snarf-constant "EALREADY")
(snarf-constant "ECONNRESET")

(define ##socket-make
  (c-lambda (int int int) (pointer "scm_socket") "new_scm_socket"))

(define ##socket-free
  (c-lambda ((pointer "scm_socket")) void "free"))

(define ##socket-fd
  (c-lambda ((pointer "scm_socket")) int "___result = ___arg1->fd;"))

(define ##socket-address
  (c-lambda ((pointer "scm_socket")) char-string
"char buffer[100];
 in_address_as_string ((struct sockaddr_in *)(&(___arg1->sock_addr)), buffer);"))

(define ##socket-set-blocking
  (c-lambda ((pointer "scm_socket") boolean) void "scm_socket_set_blocking"))

(define ##socket-connect
  (c-lambda ((pointer "scm_socket") char-string) int "scm_socket_connect"))

(define ##socket-listen
  (c-lambda ((pointer "scm_socket") int) int "___result = listen (___arg1->fd, ___arg2);"))

(define ##socket-accept
  (c-lambda ((pointer "scm_socket")) (pointer "scm_socket") "scm_socket_accept"))

(define ##socket-bind
  (c-lambda ((pointer "scm_socket") char-string) int "scm_socket_bind"))

(define ##socket-send
  (c-lambda ((pointer "scm_socket") char-string int) int
 "___result = send (___arg1->fd, ___arg2, ___arg3, 0);"))

(define ##socket-recv
  (c-lambda ((pointer "scm_socket") int) char-string
 "char * buffer = alloca(___arg2+1);
 int length = recv (___arg1->fd, buffer, ___arg2, 0);
 if (length >= 0) {
   buffer[length] = (char)(0);
   ___result = buffer;
 } else {
   ___result = NULL;
 }
"))

(define ##socket-close
  (c-lambda ((pointer "scm_socket")) int "___result = close (___arg1->fd);"))

;;;
;;; file descriptor sets
;;;

;; Note: On Win32, we know enough about how the fd_set struct is used
;; that we can dynamically choose how large to make the set, without
;; requiring a size limit to be compiled into the code (via FD_SETSIZE).
;; [can we do this on Unix, too?]

(define ##fd-set-make
  (c-lambda () (pointer "fd_set") "___result = (fd_set *) malloc (sizeof(fd_set));"))

(define ##fd-set-free
  (c-lambda ((pointer "fd_set")) void "free"))

(define ##fd-set-zero
  (c-lambda ((pointer "fd_set")) void "FD_ZERO(___arg1);"))

(define ##fd-set
  (c-lambda ((pointer "fd_set") int) void "FD_SET(___arg2, ___arg1);"))

(define ##fd-isset
  (c-lambda ((pointer "fd_set") int) boolean "___result = FD_ISSET(___arg2, ___arg1);"))

;;;
;;; select() interface
;;;

;;; On Win32, the first parameter is ignored.
;;; On unix, we need to compute it.  Hmmm...

(define ##select
  (c-lambda
   (int
    (pointer "fd_set")			; read_fds
    (pointer "fd_set")			; write_fds
    (pointer "fd_set")			; except_fds
    boolean				; wait?
    float)				; timeout
   int
"struct timeval tv = { (int) ___arg5, (int) (___arg5 * 1000) };
 struct timeval * tvaddr = ___arg4 ? &tv : NULL;
 ___result = select (___arg1, ___arg2, ___arg3, ___arg4, &tv); "))

;;; support

(c-declare
 "#include <string.h>
  #include <errno.h> ")

(define ##error-as-string
  (c-lambda () char-string "___result = strerror (errno);"))

;;; on win32, we need to initialize the winsock library
(if (not ((c-lambda () boolean "scm_socket_initialize")))
    (error "WINSOCK failed to initialize"))
